https://sourceware.org/bugzilla/show_bug.cgi?id=21002

Index: elfutils-0.169/lib/system.h
===================================================================
--- elfutils-0.169.orig/lib/system.h
+++ elfutils-0.169/lib/system.h
@@ -29,7 +29,18 @@
 #ifndef LIB_SYSTEM_H
 #define LIB_SYSTEM_H	1
 
+#ifdef HAVE_ERROR_H
+#include "error.h"
+#else
+#include "err.h"
 #include <errno.h>
+#include <stdio.h>
+#define error(status, errno, ...) \
+	fflush(stdout); \
+	warn(__VA_ARGS__); \
+	if (status) exit(status)
+#endif
+
 #include <stddef.h>
 #include <stdint.h>
 #include <sys/param.h>
@@ -37,6 +48,10 @@
 #include <byteswap.h>
 #include <unistd.h>
 
+#ifndef __GLIBC__
+#define canonicalize_file_name(name) realpath(name,NULL)
+#endif
+
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 # define LE32(n)	(n)
 # define LE64(n)	(n)
Index: elfutils-0.169/lib/color.c
===================================================================
--- elfutils-0.169.orig/lib/color.c
+++ elfutils-0.169/lib/color.c
@@ -32,13 +32,13 @@
 #endif
 
 #include <argp.h>
-#include <error.h>
 #include <libintl.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include "libeu.h"
 #include "color.h"
+#include "system.h"
 
 /* Prototype for option handler.  */
 static error_t parse_opt (int key, char *arg, struct argp_state *state);
Index: elfutils-0.169/lib/xmalloc.c
===================================================================
--- elfutils-0.169.orig/lib/xmalloc.c
+++ elfutils-0.169/lib/xmalloc.c
@@ -30,7 +30,6 @@
 # include <config.h>
 #endif
 
-#include <error.h>
 #include <libintl.h>
 #include <stddef.h>
 #include <stdlib.h>
Index: elfutils-0.169/src/addr2line.c
===================================================================
--- elfutils-0.169.orig/src/addr2line.c
+++ elfutils-0.169/src/addr2line.c
@@ -23,7 +23,6 @@
 #include <argp.h>
 #include <assert.h>
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <inttypes.h>
 #include <libdwfl.h>
Index: elfutils-0.169/src/ar.c
===================================================================
--- elfutils-0.169.orig/src/ar.c
+++ elfutils-0.169/src/ar.c
@@ -22,7 +22,6 @@
 
 #include <argp.h>
 #include <assert.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <libintl.h>
Index: elfutils-0.169/src/arlib2.c
===================================================================
--- elfutils-0.169.orig/src/arlib2.c
+++ elfutils-0.169/src/arlib2.c
@@ -20,7 +20,6 @@
 # include <config.h>
 #endif
 
-#include <error.h>
 #include <libintl.h>
 #include <limits.h>
 #include <string.h>
Index: elfutils-0.169/src/arlib.c
===================================================================
--- elfutils-0.169.orig/src/arlib.c
+++ elfutils-0.169/src/arlib.c
@@ -21,7 +21,6 @@
 #endif
 
 #include <assert.h>
-#include <error.h>
 #include <gelf.h>
 #include <inttypes.h>
 #include <libintl.h>
Index: elfutils-0.169/src/elfcmp.c
===================================================================
--- elfutils-0.169.orig/src/elfcmp.c
+++ elfutils-0.169/src/elfcmp.c
@@ -23,7 +23,6 @@
 #include <argp.h>
 #include <assert.h>
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <locale.h>
 #include <libintl.h>
Index: elfutils-0.169/src/elflint.c
===================================================================
--- elfutils-0.169.orig/src/elflint.c
+++ elfutils-0.169/src/elflint.c
@@ -24,7 +24,6 @@
 #include <assert.h>
 #include <byteswap.h>
 #include <endian.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <inttypes.h>
Index: elfutils-0.169/src/findtextrel.c
===================================================================
--- elfutils-0.169.orig/src/findtextrel.c
+++ elfutils-0.169/src/findtextrel.c
@@ -23,7 +23,6 @@
 #include <argp.h>
 #include <assert.h>
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <libdw.h>
Index: elfutils-0.169/src/nm.c
===================================================================
--- elfutils-0.169.orig/src/nm.c
+++ elfutils-0.169/src/nm.c
@@ -26,7 +26,6 @@
 #include <ctype.h>
 #include <dwarf.h>
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <inttypes.h>
Index: elfutils-0.169/src/objdump.c
===================================================================
--- elfutils-0.169.orig/src/objdump.c
+++ elfutils-0.169/src/objdump.c
@@ -21,7 +21,6 @@
 #endif
 
 #include <argp.h>
-#include <error.h>
 #include <fcntl.h>
 #include <inttypes.h>
 #include <libintl.h>
Index: elfutils-0.169/src/ranlib.c
===================================================================
--- elfutils-0.169.orig/src/ranlib.c
+++ elfutils-0.169/src/ranlib.c
@@ -24,7 +24,6 @@
 #include <argp.h>
 #include <assert.h>
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <libintl.h>
Index: elfutils-0.169/src/readelf.c
===================================================================
--- elfutils-0.169.orig/src/readelf.c
+++ elfutils-0.169/src/readelf.c
@@ -25,7 +25,6 @@
 #include <ctype.h>
 #include <dwarf.h>
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <inttypes.h>
Index: elfutils-0.169/src/size.c
===================================================================
--- elfutils-0.169.orig/src/size.c
+++ elfutils-0.169/src/size.c
@@ -21,7 +21,6 @@
 #endif
 
 #include <argp.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <inttypes.h>
Index: elfutils-0.169/src/stack.c
===================================================================
--- elfutils-0.169.orig/src/stack.c
+++ elfutils-0.169/src/stack.c
@@ -18,7 +18,6 @@
 #include <config.h>
 #include <assert.h>
 #include <argp.h>
-#include <error.h>
 #include <stdlib.h>
 #include <inttypes.h>
 #include <stdio.h>
Index: elfutils-0.169/src/strings.c
===================================================================
--- elfutils-0.169.orig/src/strings.c
+++ elfutils-0.169/src/strings.c
@@ -25,7 +25,6 @@
 #include <ctype.h>
 #include <endian.h>
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <inttypes.h>
Index: elfutils-0.169/src/strip.c
===================================================================
--- elfutils-0.169.orig/src/strip.c
+++ elfutils-0.169/src/strip.c
@@ -24,7 +24,6 @@
 #include <assert.h>
 #include <byteswap.h>
 #include <endian.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <libelf.h>
Index: elfutils-0.169/src/unstrip.c
===================================================================
--- elfutils-0.169.orig/src/unstrip.c
+++ elfutils-0.169/src/unstrip.c
@@ -31,7 +31,6 @@
 #include <argp.h>
 #include <assert.h>
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <fnmatch.h>
 #include <libintl.h>
Index: elfutils-0.169/tests/addrscopes.c
===================================================================
--- elfutils-0.169.orig/tests/addrscopes.c
+++ elfutils-0.169/tests/addrscopes.c
@@ -25,7 +25,6 @@
 #include <stdio_ext.h>
 #include <locale.h>
 #include <stdlib.h>
-#include <error.h>
 #include <string.h>
 
 
Index: elfutils-0.169/tests/allregs.c
===================================================================
--- elfutils-0.169.orig/tests/allregs.c
+++ elfutils-0.169/tests/allregs.c
@@ -21,7 +21,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <error.h>
 #include <locale.h>
 #include <argp.h>
 #include <assert.h>
Index: elfutils-0.169/tests/backtrace.c
===================================================================
--- elfutils-0.169.orig/tests/backtrace.c
+++ elfutils-0.169/tests/backtrace.c
@@ -24,7 +24,6 @@
 #include <dirent.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <error.h>
 #include <unistd.h>
 #include <dwarf.h>
 #ifdef __linux__
Index: elfutils-0.169/tests/backtrace-data.c
===================================================================
--- elfutils-0.169.orig/tests/backtrace-data.c
+++ elfutils-0.169/tests/backtrace-data.c
@@ -27,7 +27,6 @@
 #include <dirent.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <error.h>
 #include <unistd.h>
 #include <dwarf.h>
 #if defined(__x86_64__) && defined(__linux__)
Index: elfutils-0.169/tests/buildid.c
===================================================================
--- elfutils-0.169.orig/tests/buildid.c
+++ elfutils-0.169/tests/buildid.c
@@ -23,7 +23,6 @@
 #include ELFUTILS_HEADER(elf)
 #include ELFUTILS_HEADER(dwelf)
 #include <stdio.h>
-#include <error.h>
 #include <string.h>
 #include <stdlib.h>
 #include <sys/types.h>
Index: elfutils-0.169/tests/debugaltlink.c
===================================================================
--- elfutils-0.169.orig/tests/debugaltlink.c
+++ elfutils-0.169/tests/debugaltlink.c
@@ -23,7 +23,6 @@
 #include ELFUTILS_HEADER(dw)
 #include ELFUTILS_HEADER(dwelf)
 #include <stdio.h>
-#include <error.h>
 #include <string.h>
 #include <stdlib.h>
 #include <sys/types.h>
Index: elfutils-0.169/tests/debuglink.c
===================================================================
--- elfutils-0.169.orig/tests/debuglink.c
+++ elfutils-0.169/tests/debuglink.c
@@ -21,7 +21,6 @@
 #include <errno.h>
 #include ELFUTILS_HEADER(dwelf)
 #include <stdio.h>
-#include <error.h>
 #include <string.h>
 #include <stdlib.h>
 #include <sys/types.h>
Index: elfutils-0.169/tests/dwfl-addr-sect.c
===================================================================
--- elfutils-0.169.orig/tests/dwfl-addr-sect.c
+++ elfutils-0.169/tests/dwfl-addr-sect.c
@@ -23,7 +23,6 @@
 #include <stdio_ext.h>
 #include <stdlib.h>
 #include <string.h>
-#include <error.h>
 #include <locale.h>
 #include <argp.h>
 #include ELFUTILS_HEADER(dwfl)
Index: elfutils-0.169/tests/dwfl-bug-addr-overflow.c
===================================================================
--- elfutils-0.169.orig/tests/dwfl-bug-addr-overflow.c
+++ elfutils-0.169/tests/dwfl-bug-addr-overflow.c
@@ -20,7 +20,6 @@
 #include <inttypes.h>
 #include <stdio.h>
 #include <stdio_ext.h>
-#include <error.h>
 #include <locale.h>
 #include ELFUTILS_HEADER(dwfl)
 
Index: elfutils-0.169/tests/dwfl-bug-fd-leak.c
===================================================================
--- elfutils-0.169.orig/tests/dwfl-bug-fd-leak.c
+++ elfutils-0.169/tests/dwfl-bug-fd-leak.c
@@ -24,7 +24,6 @@
 #include <dirent.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <error.h>
 #include <unistd.h>
 #include <dwarf.h>
 
Index: elfutils-0.169/tests/dwfl-bug-getmodules.c
===================================================================
--- elfutils-0.169.orig/tests/dwfl-bug-getmodules.c
+++ elfutils-0.169/tests/dwfl-bug-getmodules.c
@@ -18,7 +18,6 @@
 #include <config.h>
 #include ELFUTILS_HEADER(dwfl)
 
-#include <error.h>
 
 static const Dwfl_Callbacks callbacks =
   {
Index: elfutils-0.169/tests/dwfllines.c
===================================================================
--- elfutils-0.169.orig/tests/dwfllines.c
+++ elfutils-0.169/tests/dwfllines.c
@@ -27,7 +27,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <error.h>
 
 int
 main (int argc, char *argv[])
Index: elfutils-0.169/tests/dwflmodtest.c
===================================================================
--- elfutils-0.169.orig/tests/dwflmodtest.c
+++ elfutils-0.169/tests/dwflmodtest.c
@@ -23,7 +23,6 @@
 #include <stdio_ext.h>
 #include <stdlib.h>
 #include <string.h>
-#include <error.h>
 #include <locale.h>
 #include <argp.h>
 #include ELFUTILS_HEADER(dwfl)
Index: elfutils-0.169/tests/dwfl-report-elf-align.c
===================================================================
--- elfutils-0.169.orig/tests/dwfl-report-elf-align.c
+++ elfutils-0.169/tests/dwfl-report-elf-align.c
@@ -20,7 +20,6 @@
 #include <inttypes.h>
 #include <stdio.h>
 #include <stdio_ext.h>
-#include <error.h>
 #include <locale.h>
 #include <string.h>
 #include <stdlib.h>
Index: elfutils-0.169/tests/dwflsyms.c
===================================================================
--- elfutils-0.169.orig/tests/dwflsyms.c
+++ elfutils-0.169/tests/dwflsyms.c
@@ -25,7 +25,6 @@
 #include <stdio.h>
 #include <stdio_ext.h>
 #include <stdlib.h>
-#include <error.h>
 #include <string.h>
 
 static const char *
Index: elfutils-0.169/tests/early-offscn.c
===================================================================
--- elfutils-0.169.orig/tests/early-offscn.c
+++ elfutils-0.169/tests/early-offscn.c
@@ -19,7 +19,6 @@
 #endif
 
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <stdio.h>
Index: elfutils-0.169/tests/ecp.c
===================================================================
--- elfutils-0.169.orig/tests/ecp.c
+++ elfutils-0.169/tests/ecp.c
@@ -20,7 +20,6 @@
 #endif
 
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <stdlib.h>
Index: elfutils-0.169/tests/find-prologues.c
===================================================================
--- elfutils-0.169.orig/tests/find-prologues.c
+++ elfutils-0.169/tests/find-prologues.c
@@ -25,7 +25,6 @@
 #include <stdio_ext.h>
 #include <locale.h>
 #include <stdlib.h>
-#include <error.h>
 #include <string.h>
 #include <fnmatch.h>
 
Index: elfutils-0.169/tests/funcretval.c
===================================================================
--- elfutils-0.169.orig/tests/funcretval.c
+++ elfutils-0.169/tests/funcretval.c
@@ -25,7 +25,6 @@
 #include <stdio_ext.h>
 #include <locale.h>
 #include <stdlib.h>
-#include <error.h>
 #include <string.h>
 #include <fnmatch.h>
 
Index: elfutils-0.169/tests/funcscopes.c
===================================================================
--- elfutils-0.169.orig/tests/funcscopes.c
+++ elfutils-0.169/tests/funcscopes.c
@@ -25,7 +25,6 @@
 #include <stdio_ext.h>
 #include <locale.h>
 #include <stdlib.h>
-#include <error.h>
 #include <string.h>
 #include <fnmatch.h>
 
Index: elfutils-0.169/tests/line2addr.c
===================================================================
--- elfutils-0.169.orig/tests/line2addr.c
+++ elfutils-0.169/tests/line2addr.c
@@ -26,7 +26,6 @@
 #include <locale.h>
 #include <stdlib.h>
 #include <string.h>
-#include <error.h>
 
 
 static void
Index: elfutils-0.169/tests/low_high_pc.c
===================================================================
--- elfutils-0.169.orig/tests/low_high_pc.c
+++ elfutils-0.169/tests/low_high_pc.c
@@ -25,7 +25,6 @@
 #include <stdio_ext.h>
 #include <locale.h>
 #include <stdlib.h>
-#include <error.h>
 #include <string.h>
 #include <fnmatch.h>
 
Index: elfutils-0.169/tests/md5-sha1-test.c
===================================================================
--- elfutils-0.169.orig/tests/md5-sha1-test.c
+++ elfutils-0.169/tests/md5-sha1-test.c
@@ -19,7 +19,6 @@
 #endif
 
 #include <string.h>
-#include <error.h>
 
 #include "md5.h"
 #include "sha1.h"
Index: elfutils-0.169/tests/rdwrmmap.c
===================================================================
--- elfutils-0.169.orig/tests/rdwrmmap.c
+++ elfutils-0.169/tests/rdwrmmap.c
@@ -19,7 +19,6 @@
 #endif
 
 #include <errno.h>
-#include <error.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
Index: elfutils-0.169/tests/saridx.c
===================================================================
--- elfutils-0.169.orig/tests/saridx.c
+++ elfutils-0.169/tests/saridx.c
@@ -17,7 +17,6 @@
 
 #include <config.h>
 
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <stdio.h>
Index: elfutils-0.169/tests/sectiondump.c
===================================================================
--- elfutils-0.169.orig/tests/sectiondump.c
+++ elfutils-0.169/tests/sectiondump.c
@@ -18,7 +18,6 @@
 #include <config.h>
 
 #include <errno.h>
-#include <error.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <inttypes.h>
Index: elfutils-0.169/tests/varlocs.c
===================================================================
--- elfutils-0.169.orig/tests/varlocs.c
+++ elfutils-0.169/tests/varlocs.c
@@ -25,7 +25,6 @@
 #include <dwarf.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <error.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
Index: elfutils-0.169/libasm/asm_end.c
===================================================================
--- elfutils-0.169.orig/libasm/asm_end.c
+++ elfutils-0.169/libasm/asm_end.c
@@ -32,7 +32,6 @@
 #endif
 
 #include <assert.h>
-#include <error.h>
 #include <libintl.h>
 #include <stdio.h>
 #include <stdlib.h>
Index: elfutils-0.169/libasm/asm_newscn.c
===================================================================
--- elfutils-0.169.orig/libasm/asm_newscn.c
+++ elfutils-0.169/libasm/asm_newscn.c
@@ -32,7 +32,6 @@
 #endif
 
 #include <assert.h>
-#include <error.h>
 #include <libintl.h>
 #include <stdlib.h>
 #include <string.h>
Index: elfutils-0.169/libcpu/i386_gendis.c
===================================================================
--- elfutils-0.169.orig/libcpu/i386_gendis.c
+++ elfutils-0.169/libcpu/i386_gendis.c
@@ -31,7 +31,6 @@
 # include <config.h>
 #endif
 
-#include <error.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
Index: elfutils-0.169/libcpu/i386_lex.c
===================================================================
--- elfutils-0.169.orig/libcpu/i386_lex.c
+++ elfutils-0.169/libcpu/i386_lex.c
@@ -592,7 +592,6 @@ char *i386_text;
 #endif
 
 #include <ctype.h>
-#include <error.h>
 #include <libintl.h>
 
 #include <libeu.h>
Index: elfutils-0.169/libcpu/i386_lex.l
===================================================================
--- elfutils-0.169.orig/libcpu/i386_lex.l
+++ elfutils-0.169/libcpu/i386_lex.l
@@ -31,7 +31,6 @@
 #endif
 
 #include <ctype.h>
-#include <error.h>
 #include <libintl.h>
 
 #include <libeu.h>
Index: elfutils-0.169/libcpu/i386_parse.c
===================================================================
--- elfutils-0.169.orig/libcpu/i386_parse.c
+++ elfutils-0.169/libcpu/i386_parse.c
@@ -107,7 +107,6 @@
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
-#include <error.h>
 #include <inttypes.h>
 #include <libintl.h>
 #include <math.h>
Index: elfutils-0.169/libdw/libdw_alloc.c
===================================================================
--- elfutils-0.169.orig/libdw/libdw_alloc.c
+++ elfutils-0.169/libdw/libdw_alloc.c
@@ -31,7 +31,6 @@
 # include <config.h>
 #endif
 
-#include <error.h>
 #include <errno.h>
 #include <stdlib.h>
 #include "libdwP.h"
@@ -74,5 +73,5 @@ __attribute ((noreturn, visibility ("hid
 __libdw_oom (void)
 {
   while (1)
-    error (EXIT_FAILURE, ENOMEM, "libdw");
+    error (EXIT_FAILURE, errno, gettext ("cannot allocate memory"));
 }
Index: elfutils-0.169/libebl/eblopenbackend.c
===================================================================
--- elfutils-0.169.orig/libebl/eblopenbackend.c
+++ elfutils-0.169/libebl/eblopenbackend.c
@@ -32,7 +32,6 @@
 
 #include <assert.h>
 #include <dlfcn.h>
-#include <error.h>
 #include <libelfP.h>
 #include <dwarf.h>
 #include <stdlib.h>
Index: elfutils-0.169/libdwfl/dwfl_error.c
===================================================================
--- elfutils-0.169.orig/libdwfl/dwfl_error.c
+++ elfutils-0.169/libdwfl/dwfl_error.c
@@ -140,6 +140,7 @@ __libdwfl_seterrno (Dwfl_Error error)
 const char *
 dwfl_errmsg (int error)
 {
+  static __thread char s[64] = "";
   if (error == 0 || error == -1)
     {
       int last_error = global_error;
@@ -154,7 +155,8 @@ dwfl_errmsg (int error)
   switch (error &~ 0xffff)
     {
     case OTHER_ERROR (ERRNO):
-      return strerror_r (error & 0xffff, "bad", 0);
+      strerror_r (error & 0xffff, s, sizeof(s));
+      return s;
     case OTHER_ERROR (LIBELF):
       return elf_errmsg (error & 0xffff);
     case OTHER_ERROR (LIBDW):
